﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography.X509Certificates;
using System.Security.Cryptography.Pkcs;

namespace ConexionPlatinoWCF
{
    public class FirmaPKCS7
    {
        // Usar sEncoding="iso-8859-1" para platino
        private string _Encodiong;
        private X509Certificate2 _ClientCert;

        public FirmaPKCS7(X509Certificate2 ClientCert, string sEncodiong)
        {
            _Encodiong = sEncodiong;
            _ClientCert = ClientCert;
        }
        
        /// <summary>
        /// Firmando en PKCS7
        /// </summary>        
        /// <param name="msg"></param>              
        /// <returns></returns>
        public string SignMsgPKCS7(string msg,DateTime? dt)
        {
            try
            {
                if (_ClientCert == null) return null;

                if (!_ClientCert.HasPrivateKey) return null;

                if (Encoding.GetEncoding(_Encodiong) == null) return null;

                byte[] data = Encoding.GetEncoding(_Encodiong).GetBytes(msg);
                ContentInfo contentInfo = new ContentInfo(data);

                SignedCms signedCMS = new SignedCms(contentInfo);

                CmsSigner cmsSigner = new CmsSigner(SubjectIdentifierType.IssuerAndSerialNumber, _ClientCert);
                cmsSigner.IncludeOption = X509IncludeOption.EndCertOnly;
                if (dt.HasValue)
                    cmsSigner.SignedAttributes.Add(new Pkcs9SigningTime(dt.Value));
                else
                    cmsSigner.SignedAttributes.Add(new Pkcs9SigningTime());

                signedCMS.ComputeSignature(cmsSigner, false);

                byte[] encoded = signedCMS.Encode();
                return Convert.ToBase64String(encoded);
            }
            catch
            {
                return null;
            }
        }
    }
}
