﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.Security.Cryptography.X509Certificates;
using System.Xml.Linq;
using System.Runtime.Serialization;
using System.ServiceModel.Configuration;


namespace ConexionPlatinoWCF
{
    public class MensajeInspector : IDispatchMessageInspector, IClientMessageInspector
    {
        string sUsernameToken;
        string sPasswordUsernameToken;
        X509Certificate2 Certificado;
        string[] sPlatinoParamsNombre;
        string[] sPlatinoParamsValor;

        public MensajeInspector(string UsernameToken, string PasswordUsernameToken, X509Certificate2 Cert)
        {
            sUsernameToken = UsernameToken;
            sPasswordUsernameToken = PasswordUsernameToken;
            Certificado = Cert;
            sPlatinoParamsNombre = null;
            sPlatinoParamsValor = null;
        }

        public MensajeInspector(string UsernameToken, string PasswordUsernameToken, X509Certificate2 Cert, string[] PlatinoParamsNombre, string[] PlatinoParamsValor)
        {
            sUsernameToken = UsernameToken;
            sPasswordUsernameToken = PasswordUsernameToken;
            Certificado = Cert;
            sPlatinoParamsNombre = null;
            sPlatinoParamsValor = null;
            if (PlatinoParamsNombre != null && PlatinoParamsValor != null && PlatinoParamsNombre.Length == PlatinoParamsValor.Length)
            {
                sPlatinoParamsNombre = PlatinoParamsNombre;
                sPlatinoParamsValor = PlatinoParamsValor;
            }
        }

        #region IDispatchMessageInspector Members

        public object AfterReceiveRequest(ref System.ServiceModel.Channels.Message request, System.ServiceModel.IClientChannel channel, System.ServiceModel.InstanceContext instanceContext)
        {
            return null;
        }

        public void BeforeSendReply(ref System.ServiceModel.Channels.Message reply, object correlationState)
        {
            return;
        }

        #endregion

        #region IClientMessageInspector Members

        public void AfterReceiveReply(ref System.ServiceModel.Channels.Message reply, object correlationState)
        {
            reply.Properties.Remove(HttpResponseMessageProperty.Name);
            return;
        }

        public object BeforeSendRequest(ref System.ServiceModel.Channels.Message request, System.ServiceModel.IClientChannel channel)
        {
            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.LoadXml(MessageString(ref request));


            XmlNodeList List = xmlDocument.GetElementsByTagName("s:Body");
            XmlNode element = List[0];

            string content = element.OuterXml;
            x509_Autentificacion x509 = new x509_Autentificacion(sUsernameToken, sPasswordUsernameToken, Certificado, sPlatinoParamsNombre, sPlatinoParamsValor);

            MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(x509.CreateX509SoapEnvelope(content, true)));
            XmlDictionaryReader reader = XmlDictionaryReader.CreateTextReader(ms, XmlDictionaryReaderQuotas.Max);
            request = Message.CreateMessage(reader, Int32.MaxValue, MessageVersion.Soap11);

            return null;
        }

        private string MessageString(ref Message m)
        {
            // copy the message into a working buffer.
            MessageBuffer mb = m.CreateBufferedCopy(int.MaxValue);

            // re-create the original message, because "copy" changes its state.
            m = mb.CreateMessage();

            Stream s = new MemoryStream();
            XmlWriter xw = XmlWriter.Create(s);
            mb.CreateMessage().WriteMessage(xw);
            xw.Flush();
            s.Position = 0;

            byte[] bXML = new byte[s.Length];
            s.Read(bXML, 0, (int)s.Length);

            // sometimes bXML[] starts with a BOM
            if (bXML[0] != (byte)'<')
            {
                return Encoding.UTF8.GetString(bXML, 3, bXML.Length - 3);
            }
            else
            {
                return Encoding.UTF8.GetString(bXML, 0, bXML.Length);
            }
        }


        #endregion


    }

    [AttributeUsage(AttributeTargets.Class)]
    public class MensajeInspectorBehavior : Attribute, IEndpointBehavior
    {
        string sUsernameToken;
        string sPasswordUsernameToken;
        X509Certificate2 Certificado;
        string[] sPlatinoParamsNombre;
        string[] sPlatinoParamsValor;

        public MensajeInspectorBehavior(string UsernameToken, string PasswordUsernameToken, X509Certificate2 Cert, string[] PlatinoParamsNombre, string[] PlatinoParamsValor)
            : base()
        {
            sUsernameToken = UsernameToken;
            sPasswordUsernameToken = PasswordUsernameToken;
            Certificado = Cert;
            sPlatinoParamsNombre = PlatinoParamsNombre;
            sPlatinoParamsValor = PlatinoParamsValor;
        }

        #region IEndpointBehavior Members

        public void AddBindingParameters(ServiceEndpoint endpoint, System.ServiceModel.Channels.BindingParameterCollection bindingParameters)
        {
        }

        public void ApplyClientBehavior(ServiceEndpoint endpoint, System.ServiceModel.Dispatcher.ClientRuntime clientRuntime)
        {
            MensajeInspector inspector = null;
            inspector = new MensajeInspector(sUsernameToken, sPasswordUsernameToken, Certificado, sPlatinoParamsNombre, sPlatinoParamsValor);
            clientRuntime.MessageInspectors.Add(inspector);

        }

        public void ApplyDispatchBehavior(ServiceEndpoint endpoint, System.ServiceModel.Dispatcher.EndpointDispatcher endpointDispatcher)
        {
            //no es necesaria su implementación en cliente.
            MensajeInspector inspector = null;
            inspector = new MensajeInspector(sUsernameToken, sPasswordUsernameToken, Certificado, sPlatinoParamsNombre, sPlatinoParamsValor);
            endpointDispatcher.DispatchRuntime.MessageInspectors.Add(inspector);

        }

        public void Validate(ServiceEndpoint endpoint)
        {
            //no es necesaria su implementación en cliente.
        }

        #endregion

    }

    public class Security : System.ServiceModel.Channels.MessageHeader
    {
        public string Contenido;
        protected override void OnWriteStartHeader(System.Xml.XmlDictionaryWriter writer, System.ServiceModel.Channels.MessageVersion messageVersion)
        {
            writer.WriteStartElement("wsse", Name, Namespace);
            writer.WriteXmlnsAttribute("wsse", Namespace);
            writer.WriteAttributeString("s:actor", Actor);
            writer.WriteAttributeString("s:mustUnderstand", (MustUnderstand) ? "1" : "0");
        }

        protected override void OnWriteHeaderContents(System.Xml.XmlDictionaryWriter writer, System.ServiceModel.Channels.MessageVersion messageVersion)
        {
            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.LoadXml(Contenido);
            XmlNode node = xmlDocument.GetElementsByTagName("wsse:Security")[0];

            foreach (XmlNode nodoHijo in node.ChildNodes)
            {
                writer.WriteNode(nodoHijo.CreateNavigator(), true);
            }
        }

        public override string Name
        {
            get { return "Security"; }
        }

        public override bool MustUnderstand
        {
            get { return true; }
        }

        public override string Actor
        {
            get { return "http://www.gobiernodecanarias.org/Platino/Authentication/1.0"; }
        }

        public override string Namespace
        {
            get { return "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"; }
        }
    }

}


