﻿using System;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.Text;
using System.Xml;
using System.IO;

namespace ConexionPlatinoWCF.Platino
{
    public class MessageEncodingBindingElementPlatino : MessageEncodingBindingElement
    {
        MessageVersion messageVersion = MessageVersion.Soap11;
        public override MessageEncoderFactory CreateMessageEncoderFactory()
        {
            return new MessageEncoderFactoryPlatino(this.messageVersion);
        }
        public override MessageVersion MessageVersion
        {
            get
            {
                return this.messageVersion;
            }
            set
            {
                this.messageVersion = value;
            }
        }
        public override BindingElement Clone()
        {
            return this;
        }
        public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
        {
            context.BindingParameters.Add(this);
            return context.BuildInnerChannelFactory<TChannel>();
        }
        public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
        {
            context.BindingParameters.Add(this);
            return context.BuildInnerChannelListener<TChannel>();
        }
        public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
        {
            return context.CanBuildInnerChannelFactory<TChannel>();
        }
        public override bool CanBuildChannelListener<TChannel>(BindingContext context)
        {
            return context.CanBuildInnerChannelListener<TChannel>();
        }
    }
    class MessageEncoderFactoryPlatino : MessageEncoderFactory
    {
        MessageVersion messageVersion;
        MessageEncoderPlatino encoder;
        public MessageEncoderFactoryPlatino(MessageVersion messageVersion)
        {
            this.messageVersion = messageVersion;
            this.encoder = new MessageEncoderPlatino(messageVersion);
        }
        public override MessageEncoder Encoder
        {
            get { return this.encoder; }
        }
        public override MessageVersion MessageVersion
        {
            get { return this.encoder.MessageVersion; }
        }
    }
    class MessageEncoderPlatino : MessageEncoder
    {
        MessageEncoder textEncoder;
        MessageEncoder mtomEncoder;
        public MessageEncoderPlatino(MessageVersion messageVersion)
        {
            MtomMessageEncodingBindingElement binding = new MtomMessageEncodingBindingElement(messageVersion, Encoding.UTF8);
            binding.MaxBufferSize = 2147483647;
            binding.ReaderQuotas.MaxStringContentLength = 2147483647;
            binding.ReaderQuotas.MaxBytesPerRead = 2147483647;
            binding.ReaderQuotas.MaxDepth = 1024;
            binding.ReaderQuotas.MaxArrayLength = 2147483647;
            binding.ReaderQuotas.MaxNameTableCharCount = 2147483647;

            this.textEncoder = new TextMessageEncodingBindingElement(messageVersion, Encoding.UTF8).CreateMessageEncoderFactory().Encoder;
            this.mtomEncoder = binding.CreateMessageEncoderFactory().Encoder;
        }
        public override string ContentType
        {
            get { return this.textEncoder.ContentType; }
        }
        public override string MediaType
        {
            get { return this.textEncoder.MediaType; }
        }
        public override MessageVersion MessageVersion
        {
            get { return this.textEncoder.MessageVersion; }
        }
        public override bool IsContentTypeSupported(string contentType)
        {
            return this.mtomEncoder.IsContentTypeSupported(contentType);
        }
        public override T GetProperty<T>()
        {
            T result = this.textEncoder.GetProperty<T>();
            if (result == null)
            {
                result = this.mtomEncoder.GetProperty<T>();
            }

            if (result == null)
            {
                result = base.GetProperty<T>();
            }

            return result;
        }

        public override Message ReadMessage(ArraySegment<byte> buffer, BufferManager bufferManager, string contentType)
        {
            if (contentType != null && contentType.Contains("multipart/related"))
            {
                if (buffer.Array[0] == 10)
                {
                    ArraySegment<byte> buffernew = new ArraySegment<byte>(buffer.Array, 1, buffer.Count);
                    buffernew.Array[0] = 13;
                    return this.mtomEncoder.ReadMessage(buffernew, bufferManager, contentType);
                }
                else
                    return this.mtomEncoder.ReadMessage(buffer, bufferManager, contentType);
            }

            return this.textEncoder.ReadMessage(buffer, bufferManager, contentType);
        }
        public override Message ReadMessage(Stream stream, int maxSizeOfHeaders, string contentType)
        {
            return this.mtomEncoder.ReadMessage(stream, maxSizeOfHeaders, contentType);
        }
        public override ArraySegment<byte> WriteMessage(Message message, int maxMessageSize, BufferManager bufferManager, int messageOffset)
        {
            return this.textEncoder.WriteMessage(message, maxMessageSize, bufferManager, messageOffset);
        }
        public override void WriteMessage(Message message, Stream stream)
        {
            this.textEncoder.WriteMessage(message, stream);
        }
    }
}
