﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography.X509Certificates;
using System.Xml;
using System.Security.Cryptography.Xml;
using System.Security.Cryptography;
using System.Reflection;
using System.IO;

namespace ConexionPlatinoWCF
{
    public class x509_Autentificacion
    {
        string sUsernameToken;
        string sPasswordUsernameToken;
        X509Certificate2 Certificado;
        string[] sPlatinoParamsNombre;
        string[] sPlatinoParamsValor;

        public x509_Autentificacion(string UsernameToken, string PasswordUsernameToken, X509Certificate2 Cert, string[] PlatinoParamsNombre, string[] PlatinoParamsValor)
        {
            sUsernameToken = UsernameToken;
            sPasswordUsernameToken = PasswordUsernameToken;
            Certificado = Cert;
            sPlatinoParamsNombre = PlatinoParamsNombre;
            sPlatinoParamsValor = PlatinoParamsValor;
        }
        public string CreateX509SoapEnvelope(string xml,bool bSingBody)
        {
            string soapXML;

            soapXML = "<s:Envelope xmlns:xop=\"http://www.w3.org/2004/08/xop/include\" xmlns:sfst=\"http://platino.gobcan.es/servicios/sfst/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"  xmlns:wsse=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd\" xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:u=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\">";                

            soapXML += "<s:Header>\n";
            
            string sIdSecurityToken = "SecurityToken-" + Guid.NewGuid().ToString();
            string sIdUsernameToken = "UsernameToken-" + Guid.NewGuid().ToString();
            string sIdPlatino = null;

            // Añadimos security block para X.509 certificado
            soapXML += "<wsse:Security xmlns:wsse=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd\" s:actor=\"http://www.gobiernodecanarias.org/Platino/Authentication/1.0\" s:mustUnderstand=\"1\" >";

            string scertificado = Convert.ToBase64String(Certificado.RawData);
            soapXML += "<wsse:BinarySecurityToken xmlns:wsu=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\" EncodingType=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary\" ValueType=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3\" wsu:Id=\"" + sIdSecurityToken + "\">" + scertificado + "</wsse:BinarySecurityToken>";
            soapXML += WriteUserNameToken(sIdUsernameToken, sUsernameToken, sPasswordUsernameToken);

            
            soapXML += "</wsse:Security>";

            // Cabeceras platino           
            if (sPlatinoParamsNombre != null && sPlatinoParamsNombre.Length > 0)
            {
                string sId = "PlatinoHeader-" + Guid.NewGuid().ToString();
                string sPlatino = WriteParametrosPlatino(sId, sPlatinoParamsNombre, sPlatinoParamsValor);

                if (!string.IsNullOrEmpty(sPlatino))
                {
                    soapXML += sPlatino;
                    sIdPlatino = sId;
                }
            }
            else
                sIdPlatino = null;

            soapXML += "</s:Header>" + xml + "</s:Envelope>";

            // Sign Envelope
            soapXML = CreateSignatureBlock(soapXML, "wsse:Security", sIdSecurityToken, sIdUsernameToken, sIdPlatino, bSingBody);

            // Verificamos que  XML esta firmado adecuadamente
            VerifySignedXml(soapXML);
            return soapXML;
        }

        public string CreateSignatureBlock(string xml, string sParentSignatureTagName, string sIdSecurityToken, string sIdUsernameToken, string sIdPlatino, bool bSingBody)
        {
            try
            {
                string sIdBody = "Body-" + Guid.NewGuid().ToString();
        
                //Cargamos Xml
                XmlDocument xd = new XmlDocument();
                xd.LoadXml(xml);

                if (bSingBody)
                {
                    XmlNodeList List = xd.GetElementsByTagName("s:Body");
                    XmlNode element = List[0];

                    // Asignamos id a Body
                    try
                    {
                        if (element.Attributes.GetNamedItem("wsu:Id").Value == null) { }
                    }
                    catch (Exception)
                    {
                        XmlAttribute atrId = xd.CreateAttribute("wsu", "Id", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd");
                        atrId.Value = sIdBody;
                        element.Attributes.Append(atrId);
                    }
                }
                // Firmamos por id
                SignedXmlConId signedXml = new SignedXmlConId(xd);
                signedXml.SigningKey = Certificado.PrivateKey;

                // Creamos new KeyInfo object.
                KeyInfo keyInfo = new KeyInfo();
                XmlDocument doc = new XmlDocument();

                doc.LoadXml("<wsse:SecurityTokenReference xmlns:wsse=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd\" xmlns:wsu=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\"><wsse:Reference URI=\"#" + sIdSecurityToken + "\" ValueType=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3\"/></wsse:SecurityTokenReference>");
                keyInfo.AddClause(new KeyInfoNode(doc.DocumentElement));
                signedXml.KeyInfo = keyInfo;

                // Necesario uso de External Canonicalization metodo
                signedXml.SignedInfo.CanonicalizationMethod = SignedXml.XmlDsigExcC14NTransformUrl;

                //Creamos reference a el UsernameToken para la firma
                Reference reference = new Reference("#" + sIdUsernameToken);

                // Añadimos transformación de la reference.
                XmlDsigExcC14NTransform env = new XmlDsigExcC14NTransform();
                reference.AddTransform(env);
                reference.DigestMethod = "http://www.w3.org/2000/09/xmldsig#sha1";

                // Añadimos reference a SignedXml object.
                signedXml.AddReference(reference);
                
                //Creamos reference a el Body para la firma
                if (bSingBody)
                {
                    Reference reference2 = new Reference("#" + sIdBody);

                    // Añadimos transformación de la reference.
                    XmlDsigExcC14NTransform env2 = new XmlDsigExcC14NTransform();
                    reference2.AddTransform(env2);
                    reference2.DigestMethod = "http://www.w3.org/2000/09/xmldsig#sha1";

                    // Añadimos reference a SignedXml object.
                    signedXml.AddReference(reference2);
                }

                //Creamos reference a el Body para la firma
                if (sIdPlatino != null)
                {
                    Reference reference3 = new Reference("#" + sIdPlatino);

                    // Añadimos transformación de la reference.
                    XmlDsigExcC14NTransform env3 = new XmlDsigExcC14NTransform();
                    reference3.AddTransform(env3);
                    reference3.DigestMethod = "http://www.w3.org/2000/09/xmldsig#sha1";

                    // Añadimos reference a SignedXml object.
                    signedXml.AddReference(reference3);
                }

                // Añadimos Signature Id
                signedXml.Signature.Id = "";

                // Calculamos la signature.
                signedXml.ComputeSignature();

                // Get the XML representation of the signature and save
                // it to an XmlElement object.
                XmlElement xmlDigitalSignature = signedXml.GetXml();

                // Añadimos la signature y reordenamos
                XmlNodeList nodeList = xd.GetElementsByTagName(sParentSignatureTagName);
                if (nodeList.Count > 0)
                {
                    XmlNode headerNode = nodeList[0];
                    headerNode.AppendChild(xd.ImportNode(xmlDigitalSignature, true));

                    // Movemos el UsernameToken
                    XmlNode UsernameTokenNode = headerNode["wsse:UsernameToken"];
                    if (UsernameTokenNode != null)
                    {
                        XmlNode UsernameTokenNodeClone = UsernameTokenNode.Clone();
                        headerNode.RemoveChild(UsernameTokenNode);
                        headerNode.AppendChild(UsernameTokenNodeClone);
                    }

                    // Movemos el platino Header
                    //XmlNode PlatinoNode = headerNode["platino:platinoHeaders"];
                    //if (PlatinoNode != null)
                    //{
                    //    XmlNode PlatinoNodeClone = PlatinoNode.Clone();
                    //    headerNode.RemoveChild(PlatinoNode);
                    //    headerNode.AppendChild(PlatinoNodeClone);
                    //}

                }

                return xd.InnerXml;
            }
            catch (Exception excp)
            {
                return xml;
            }
        }

        public void VerifySignedXml(String xml)
        {
            // Creamos new XML document.
            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.PreserveWhitespace = true;
            xmlDocument.LoadXml(xml);

            // Creamos new SignedXml object 
            SignedXml signedXml = new SignedXml(xmlDocument);

            // Añadimos enveloped transformación a reference.
            XmlDsigExcC14NTransform env = new XmlDsigExcC14NTransform();

            Reference reference = new Reference();
            reference.AddTransform(env);
            signedXml.AddReference(reference);

            // Buscamos "Signature" node y creamos a new XmlNodeList object.
            XmlNodeList nodeList = xmlDocument.GetElementsByTagName("Signature");

            // Cargamos el signature node.
            signedXml.LoadXml((XmlElement)nodeList[0]);

        }


        public String WriteUserNameToken(string Id, string UserName, string Password)
        {
            Random r = new Random();
            string tokennamespace = "wsse";
            DateTime created = DateTime.Now;
            string createdStr = created.ToString("yyyy-MM-ddThh:mm:ssZ");

            byte[] nonce_bytes = new byte[16];
            RandomNumberGenerator rndGenerator = new RNGCryptoServiceProvider();
            rndGenerator.GetBytes(nonce_bytes);
            string nonce = Convert.ToBase64String(nonce_bytes);

            return String.Format(
            "<{0}:UsernameToken wsu:Id=\"" + Id + "\" xmlns:wsu=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\">" +
            "<{0}:Username>" + UserName + "</{0}:Username>" +
            "<{0}:Password Type=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText\">" +
            Password + "</{0}:Password>" +
            "<{0}:Nonce>" +
            nonce + "</{0}:Nonce>" +
            "<wsu:Created>" + createdStr + "</wsu:Created></{0}:UsernameToken>", tokennamespace);
        }

        public String WriteParametrosPlatino(string Id,string[] NombreParametrosPlatino, string[] ValorParametrosPlatino)
        {
            String s = "<platino:platinoHeaders xmlns:platino=\"http://platino.gobcan.es\" xmlns:wsu=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\" wsu:Id=\"" + Id + "\" >";

            // Creamos los datos de platino
            for (short i = 0; i < NombreParametrosPlatino.Length; i++)
            {
                s += "<platino:parameter name=\"" + NombreParametrosPlatino[i] + "\">" + ValorParametrosPlatino[i] + "</platino:parameter>";

            }
            s += "</platino:platinoHeaders>";

            return s;
        }

    }
}
